<?php
	/**
	 *	eBag
	 *
	 *	Open source company administration manager
	 *
	 *	@package	eBag
	 *	@author		Eper Kalman
	 *	@copyright	Copyright (c) 2012-2015, Eper Kalman
	 *	@license	GNU Affero General Public License Version 3 (http://www.gnu.org/licenses/agpl-3.0.html)
	 *	@since		Version 5.0.0
	 */

	/**
	 * eBag new item to order to stock
	 */
	 
	if (! defined('eBag'))	{
	
	      include('../views/logout.php'); 
	      
	      exit();
	}

	load_f('is_real_positive');

	load_f('is_int_positive');


	class ADD2ORDER2STOCK	{

		function __construct()	{

			global $__SQL, $__CONF;

			$this->SQL		=	&$__SQL;

			$this->CONF		=	&$__CONF;

			$this->BASE		=	$_SESSION['BASE'];

			$this->STOCKBASE	=	'ebagstock';

			$this->UNIT		=	new UNITloader($__CONF->get('cache', 'use_apc_cache'));
		}

		private function lock()		{

			$this->SQL->query("LOCK TABLES 	$this->BASE.addressbook WRITE, 
							$this->BASE.orders_to_stock_header WRITE , 
							$this->BASE.orders_to_stock_data WRITE,
							ebagstock.finished_goods_stock WRITE, 
							ebagstock.feedstock WRITE, 
							ebag.companydata WRITE	");
		}

		private function unlock()	{

			$this->SQL->query('UNLOCK TABLES');
		}

		public function insert($CODE, $type, $quantity)	{

			# set & read essentials

			$exchange_rate	=	$this->SQL->n_escape($this->CONF->get('payments', 'exchange_rate'));

			$currency	=	$this->SQL->n_escape($this->CONF->get('payments', 'currency'));

			$this->SQL->transaction();
		
			$this->lock();

			$this->SQL->query("SELECT MAX(`personID`) AS `personID` FROM ebag.companydata");

			if ($this->SQL->num_rows())	{

				$RESULT		=	$this->SQL->RESULT->fetch_row();
				$companyID	=	"'".$RESULT[0]."'";

			}	else	  {

				$this->SQL->rollback();

				$this->unlock();

				return 'nocompanyID';
			}

			# set item type 

			$TYPES		=	array(	1	=>	'finished_goods_stock',
							2	=>	'feedstock'	);

			$NAMES		=	array(	1	=>	'articalname',
							2	=>	'feedstockname'	);

			$ITEMTYPE	=	array(	1	=>	'a',
							2	=>	'f'	);

			$STOCKTABLE 	=	$TYPES[$type];

			$STOCKNAME	=	$NAMES[$type];

			$itemtype	=	$this->SQL->n_escape($ITEMTYPE[$type]);

			# check the item is not deleted in meantime

			$code		=	$this->SQL->n_escape($CODE);

			$this->SQL->query("	SELECT `supplierID`, `unit` 
						FROM ebagstock.$STOCKTABLE
						WHERE `code`=$code AND `origin`!='1' AND `deleted` IS NOT NULL	");

			if (! $this->SQL->num_rows())	{

				$this->SQL->rollback();

				$this->unlock();

				return 'itemdeletedinmeantime';
			}

			$RESULT		=	$this->SQL->RESULT->fetch_row();

			$supplierID 	=	$RESULT[0];

			$unit		=	$RESULT[1];

			# check user input

			if ($this->UNIT->is_int_unit($unit))	{

				if (! is_int_positive($quantity)) $errtxt = 'usererror';
						
			}	else	 {

				if (! is_real_positive($quantity)) $errtxt = 'usererror';
			}

			if (isset($errtxt))	{

				$this->SQL->rollback();

				$this->unlock();

				return $errtxt;
			}

			# check for opened order (also can be changed in meantime)

			$order2stockID	=	FALSE;

			$this->SQL->query("	SELECT `personID` 
						FROM $this->BASE.orders_to_stock_header
						WHERE `supplierID`='$supplierID' AND `ordered`='0' AND `deleted` IS NOT NULL LIMIT 1	");

			if ($this->SQL->num_rows())	{

				$RESULT		=	$this->SQL->RESULT->fetch_row();

				$order2stockID	=	$RESULT[0];
			}

			if (! $order2stockID)	{

				# create new one

				$this->SQL->query("	SELECT 	`contactname`, `country`, `city`, `zip`, `street`, `account1`,
								`bank1`, `account2`, `bank2` FROM $this->BASE.addressbook
				  			WHERE `personID`='$supplierID' AND `deleted` IS NOT NULL LIMIT 1	");

				if (! $this->SQL->num_rows())	{

					$this->SQL->rollback();

					$this->unlock();

					return 'err_supplierdeleted->personID->'.$supplierID;
				}

				$RESULT	=	$this->SQL->RESULT->fetch_assoc();

				foreach ($RESULT as $key=>$val) $RESULT[$key] = $this->SQL->nr_escape($val);

				extract($RESULT);

				$this->SQL->query("	INSERT INTO $this->BASE.orders_to_stock_header 
							VALUES
							(NULL, $companyID, '$supplierID', $contactname, $country, $city, $zip, $street, 
							$account1, $bank1, $account2, $bank2, NULL, $currency, $exchange_rate, 
							NULL, '0', '0', NULL, NULL, '0')	");

				$order2stockID	=	$this->SQL->lastid();
			}
			
			# check for this item in data

			$this->SQL->query("	SELECT `personID` 
						FROM $this->BASE.orders_to_stock_data
						WHERE `code`=$code AND `type`=$itemtype AND `order2stockID`='$order2stockID' AND `deleted` IS NOT NULL LIMIT 1	");

			if ($this->SQL->num_rows())	{

				# add quantity

				$this->SQL->query("	UPDATE $this->BASE.orders_to_stock_data 
							SET quantity=quantity+$quantity
							WHERE `code`=$code AND `type`=$itemtype AND `order2stockID`='$order2stockID' AND `deleted` IS NOT NULL LIMIT 1	");

			}	else	  {

				# insert new one

				$this->SQL->query("	SELECT `$STOCKNAME` AS `name`, `unit`, `purchase_price`  
							FROM ebagstock.$STOCKTABLE 
							WHERE `code`=$code AND `deleted` IS NOT NULL LIMIT 1	");

				# note: must be row, checked before 

				$RESULT	=	$this->SQL->RESULT->fetch_assoc();
				
				# note: very important to change null price to 0 because of backward compatibility
				#	the column cannot be null and readed price can be null in previous versions
				
				if (! $RESULT['purchase_price']) $RESULT['purchase_price'] = '0';

				foreach ($RESULT as $key=>$val) $RESULT[$key] = $this->SQL->nr_escape($val);

				$unit	=	$this->SQL->nr_escape($unit);

				extract($RESULT);

				$this->SQL->query("	INSERT INTO $this->BASE.orders_to_stock_data
							VALUES
							(NULL, '$order2stockID', $code, $name, '$quantity',
							$unit, $purchase_price, NULL, NULL, NULL, '0', $itemtype, '0')	");
			}

			# update stock

			$this->SQL->query("	UPDATE ebagstock.$STOCKTABLE
						SET quantity_on_opened_order_to_stock=quantity_on_opened_order_to_stock+$quantity
						WHERE `code`=$code AND `deleted` IS NOT NULL LIMIT 1	");

			$this->SQL->commit();

			$this->unlock();

			return TRUE;	
		}
	}

?>
