<?php
	/**
	 *	eBag
	 *
	 *	Open source company administration manager
	 *
	 *	@package	eBag
	 *	@author		Eper Kalman
	 *	@copyright	Copyright (c) 2012-2015, Eper Kalman
	 *	@license	GNU Affero General Public License Version 3 (http://www.gnu.org/licenses/agpl-3.0.html)
	 *	@since		Version 6.0.0
	 */

	/**
	 * eBag ordered enter to stock partially
	 */
	 
	if (! defined('eBag'))	{
	
	      include('../views/logout.php'); 
	      
	      exit();
	}
	
	load_f('is_real_positive');

	load_f('is_int_positive');
	
	class ENTERBYORDERPART	{
	
		function __construct()	{

			global $__SQL, $__CONF;

			$this->SQL		=	&$__SQL;

			$this->BASE		=	$_SESSION['BASE'];
			
			$this->CONF		=	&$__CONF;

			$this->UNIT		=	new UNITloader($__CONF->get('cache', 'use_apc_cache'));

			$this->STOCKBASE	=	'ebagstock';
			
			$this->PARTIALLY	=	FALSE;
			
			$this->personIDs	=	FALSE;
		}
		
		private function lock()	{

			$this->SQL->query("	LOCK TABLES	$this->BASE.orders_to_stock_header WRITE, 
								$this->BASE.orders_to_stock_data WRITE,
								$this->STOCKBASE.finished_goods_stock WRITE, 
								$this->BASE.finished_goods_stock_enter WRITE, 
								$this->STOCKBASE.feedstock WRITE, 
								$this->BASE.feedstock_enter WRITE	");
		}

		private function unlock()	{

			$this->SQL->query('UNLOCK TABLES');
		}
		
		public function write($DATA)	{
		
			extract($DATA);
			
			if (! $order2stockID) return 'unexistentorder2stock';
			
			if ((! isset($selectedIDs)) or (! $selectedIDs) or (! count($personIDs)) or (! count($quantities))) return 'nothingisselected';
		
			if ((! $bill_number) && (! $delivery_list_number)) return 'usererror';
			
			$order2stockID		=	$this->SQL->escape($order2stockID);
			
			$bill_number		=	$this->SQL->n_escape($bill_number);

			$delivery_list_number	=	$this->SQL->n_escape($delivery_list_number);
			
			unset($DATA);
			
			$now		=	date("Y-m-d");

			$username	=	$this->SQL->escape($_SESSION['username']);

			$this->SQL->transaction();
		
			$this->lock();

			# verify the order exists, is ordered (closed) 

			$this->SQL->query("	SELECT `date_of_order`, `comment`
						FROM $this->BASE.orders_to_stock_header
				  		WHERE  `personID`='$order2stockID' AND `deleted` IS NOT NULL LIMIT 1	");

			if (! $this->SQL->num_rows())	{

				$this->SQL->rollback();

				$this->unlock();

				return 'unexistentorder2stock';
			}

			$RESULT	= $this->SQL->RESULT->fetch_assoc();

			if (! $RESULT['date_of_order'])	$errtxt = 'unclosedorder2stock';
			
			$comment = $this->SQL->r_escape($RESULT['comment']);
			
			# verify there is not delivered items on order
			# leave the posibility to deliver items more times
			
			$this->SQL->query("	SELECT `personID`
						FROM $this->BASE.orders_to_stock_data
			  			WHERE  `order2stockID`='$order2stockID' AND `date_of_delivery` IS NULL AND `deleted` IS NOT NULL ");
			  			
			if (! $this->SQL->num_rows()) $errtxt = 'deliveredonceorder2stock';

			if (isset($errtxt))	{

				$this->SQL->rollback();

				$this->unlock();

				return $errtxt;
			}
			
			# note: type stores in which stock is (a=>finished_goods_stock, f=>feedstock)
			
			$nothing_delivered = TRUE;

			$STOCKS	=	array(	'a'	=>	'finished_goods_stock',
						'f'	=>	'feedstock'	);
						
			$count = count($personIDs);
			
			for ($i=0; $i<$count; $i++)	{
			
				$personID = $this->SQL->escape($personIDs[$i]);
				
				# step over no quantities
				
				if (! $quantities[$i]) continue;
				
				$quantity = $this->SQL->escape($quantities[$i]);
			
				if (! in_array($personID, $selectedIDs)) continue;
				
				$this->SQL->query("	SELECT `code`, `quantity`, `unit`, `type`
							FROM $this->BASE.orders_to_stock_data
							WHERE  `personID`='$personID' AND `order2stockID`='$order2stockID' AND `date_of_delivery` IS NULL AND `deleted` IS NOT NULL LIMIT 1");
				
				# if is delivered step over
				
				if (! $this->SQL->num_rows()) continue;
				
				$RESULT = $this->SQL->RESULT->fetch_assoc();
				
				$STOCK		=	$STOCKS[$RESULT['type']];
				
				$code2comment	=	$RESULT['code'];

				$code		=	$this->SQL->nr_escape($RESULT['code']);
				
				$quantity_on_order =	$RESULT['quantity'];
						
				if ($this->UNIT->is_int_unit($RESULT['unit']))	{

					if (! is_int_positive($quantity)) $usererror = TRUE;
						
				}	else	 {

					if (! is_real_positive($quantity)) $usererror = TRUE;
				}
				
				if (isset($usererror)) break;

				$nothing_delivered = FALSE;

				# update the stock with the ordered quantity

				$this->SQL->query("	UPDATE $this->STOCKBASE.$STOCK
							SET	quantity_ordered_to_stock=quantity_ordered_to_stock-$quantity_on_order,
								quantity=quantity+$quantity
							WHERE `code`=$code AND `deleted` IS NOT NULL LIMIT 1	");

				# update stock enter with real quantity

				$STOCKENTER = $STOCK.'_enter';

				$this->SQL->query("	INSERT INTO $this->BASE.$STOCKENTER
							VALUES 
							(NULL, $delivery_list_number, $bill_number, '$order2stockID', $code, '$quantity', '$now', '$username', '0')	");

				# update order to stock set to delivered

				$this->SQL->query("	UPDATE $this->BASE.orders_to_stock_data
							SET	`date_of_delivery`='$now',
								`delivery_list_number`=$delivery_list_number,
								`bill_number`=$bill_number
							WHERE `personID`='$personID' AND `order2stockID`='$order2stockID' AND `deleted` IS NOT NULL LIMIT 1");
							
				# Set comment if number of delivered items >< ordered items
				
				$difference = $quantity_on_order-$quantity;
				
				if ($difference > 0) {$difference = "-$difference";} else {$difference = '+'.abs($difference);}
				
				if ($quantity != $quantity_on_order) $comment .= $this->SQL->escape("\n$code2comment -> $quantity ($difference)");
			}
			
			if (isset($usererror) or $nothing_delivered)	{
			
				$this->SQL->rollback();

				$this->unlock();

				return 'usererror';
			}
			
			# if something is delivered then on order must be one (or more) item
			
			# check after change in which state is
			
			# states 0->not delivered; 1->delivered; 2->partially delivered
				
			$this->SQL->query("	SELECT `personID` 
						FROM $this->BASE.orders_to_stock_data 
						WHERE `order2stockID`='$order2stockID' AND `date_of_delivery` IS NULL AND `deleted` IS NOT NULL");
						
			if ($this->SQL->num_rows())	{
			
				$this->SQL->query("	SELECT `personID` 
							FROM $this->BASE.orders_to_stock_data 
							WHERE `order2stockID`='$order2stockID' AND `date_of_delivery` IS NOT NULL AND `deleted` IS NOT NULL ");
							
				if ($this->SQL->num_rows())	{$STATE = 2;} 	 else	{$STATE = 0;}
			
			}	else	  {
			
				$STATE = 1;
			}
			
			# update order to stock header to

			$this->SQL->query("	UPDATE $this->BASE.orders_to_stock_header
						SET `delivered`='$STATE', `comment`='$comment'
						WHERE `personID`='$order2stockID' AND `deleted` IS NOT NULL LIMIT 1	");
						
			$this->SQL->commit();

			$this->unlock();

			return TRUE;
		}
	}

?>
