<?php
	/**
	 *	eBag
	 *
	 *	Open source company administration manager
	 *
	 *	@package	eBag
	 *	@author		Eper Kalman
	 *	@copyright	Copyright (c) 2012-2015, Eper Kalman
	 *	@license	GNU Affero General Public License Version 3 (http://www.gnu.org/licenses/agpl-3.0.html)
	 *	@since		Version 3.0.0
	 */

	/**
	 * eBag service feedstock request write extends read
	 */

	load_f('is_real_positive');

	load_f('is_int_positive');

	class FEEDSTOCKREQUESTWRITE	{

		function __construct()	{

			global $__SQL, $__CONF;

			$this->SQL		=	&$__SQL;

			$this->UNIT		=	new UNITloader($__CONF->get('cache', 'use_apc_cache'));

			$this->STOCKBASE	=	'ebagstock';
		}

		private function prepare($DATA)	{

			$CODES		=	$DATA['code'];
			$NAMES		=	$DATA['name'];
			$QUANTITYS	=	$DATA['quantity'];
			$UNITS		=	$DATA['unit'];

			$LENGTH	=	count($CODES);

			if (! $LENGTH) return 'usererror';

			$RETURN	=	array();

			for ($i=0; $i<$LENGTH; $i++)	{

				$code		=	$CODES[$i];
				$name		=	$NAMES[$i];
				$quantity	=	$QUANTITYS[$i];
				$unit		=	$UNITS[$i];

				# empty rows

				if ((is_empty($code)) && (is_empty($name)) && (is_empty($quantity)) && (is_empty($unit))) continue;
				
				# Initialize vars
				
				if (! isset($RETURN['code'])) 		$RETURN['code'] = array();
				if (! isset($RETURN['name'])) 		$RETURN['name'] = array();
				if (! isset($RETURN['quantity'])) 	$RETURN['quantity'] = array();
				if (! isset($RETURN['unit'])) 		$RETURN['unit'] = array();

				# user input

				if ((is_empty($code)) or (is_empty($name))) $RETURN = 'usererror';

				if ($this->UNIT->is_int_unit($unit))	{

					if (! is_int_positive($quantity)) $RETURN = 'usererror';
						
				}	else	 {

					if (! is_real_positive($quantity)) $RETURN = 'usererror';
				}				

				if ($RETURN == 'usererror') break;

				# feedstock exists

				$code	=	$RETURN['code'][]	=	$this->SQL->n_escape($code);

				$this->SQL->query("	SELECT `personID` 
							FROM $this->STOCKBASE.feedstock 
							WHERE `code`=$code AND `deleted` IS NOT NULL LIMIT 1	");

				if (! $this->SQL->num_rows()) {$RETURN = 'feedstocknotexist'; break;}

				$RETURN['name'][]	=	$this->SQL->n_escape($name);
				$RETURN['quantity'][]	=	$this->SQL->n_escape($quantity);
				$RETURN['unit'][]	=	$this->SQL->n_escape($unit);
			}
	
			if ((is_array($RETURN)) && (! count($RETURN))) $RETURN = 'usererror';

			return $RETURN;			
		}

		private function lock()	{

			$this->SQL->query("LOCK TABLES	$this->STOCKBASE.services WRITE, 
							$this->STOCKBASE.feedstock_requests_list WRITE, 
							$this->STOCKBASE.feedstock WRITE	");
		}

		private function unlock()	{

			$this->SQL->query("UNLOCK TABLES");
		}

		public function write($SERVICEID, $DATA)	{

			$SERVICEID	=	$this->SQL->escape($SERVICEID);

			$this->SQL->transaction();
		
			$this->lock();

			$DATA = $this->prepare($DATA);

			if ((! isset($DATA['code'])) or (! $LENGTH = count($DATA['code']))) $DATA = 'usererror';	

			if (! is_array($DATA)) 	{

				$this->SQL->rollback();

				$this->unlock();

				return $DATA;
			}
			
			# check existens
			
			$this->SQL->query("	SELECT `personID` 
						FROM $this->STOCKBASE.services
						WHERE `personID`='$SERVICEID' AND `deleted` IS NOT NULL	");
						
			if (! $this->SQL->num_rows()) {
			
				$this->SQL->rollback();

				$this->unlock();

				return 'unexistentservice';
			}

			# delete the old requests

			$this->SQL->query("	DELETE FROM $this->STOCKBASE.feedstock_requests_list 
						WHERE `serviceID`='$SERVICEID'	");

			# write new one

			for ($i=0; $i<$LENGTH; $i++)	{

				$code		=	$DATA['code'][$i];
				$name		=	$DATA['name'][$i];
				$quantity	=	$DATA['quantity'][$i];
				$unit		=	$DATA['unit'][$i];

				$this->SQL->query("	SELECT `personID` 
							FROM $this->STOCKBASE.feedstock_requests_list 
							WHERE `code`=$code AND `serviceID`='$SERVICEID' AND `deleted` IS NOT NULL LIMIT 1	");

				if ($this->SQL->num_rows())	{
	
					# update the quantity
					
					$quantity	=	str_replace("'", '', $quantity);					

					$this->SQL->query("	UPDATE $this->STOCKBASE.feedstock_requests_list 
								SET quantity=quantity+$quantity
								WHERE `code`=$code AND `serviceID`='$SERVICEID' AND `deleted` IS NOT NULL LIMIT 1");

				}	else	   {

					# insert new one
					$this->SQL->query("	INSERT INTO $this->STOCKBASE.feedstock_requests_list 
								VALUES  
								(NULL, '$SERVICEID', $code, $name, $unit, $quantity, '0')"	);

				}
			}

			$this->SQL->commit();

			$this->unlock();

			return TRUE;
		}


		public function update($SERVICEID, $DATA)	{

			# update is write to

			$RESULT = $this->write($SERVICEID, $DATA);

			return $RESULT;
		}
	}

?>
