<?php
	/**
	 *	eBag
	 *
	 *	Open source company administration manager
	 *
	 *	@package	eBag
	 *	@author		Eper Kalman
	 *	@copyright	Copyright (c) 2012-2015, Eper Kalman
	 *	@license	GNU Affero General Public License Version 3 (http://www.gnu.org/licenses/agpl-3.0.html)
	 *	@since		Version 5.0.0
	 */

	/**
	 * eBag order update
	 */

	if (! defined('eBag'))	{
	
	      include('../views/logout.php'); 
	      
	      exit();
	}
	 
	load_f('is_real_positive');

	load_f('is_int_positive');

	class ORDERUPDATE	{

		var 	$ORDER_TO_STOCK		=	FALSE;

		function __construct()	{

			global $__SQL, $__CONF;

			$this->SQL		=	&$__SQL;

			$this->CONF		=	&$__CONF;

			$this->UNIT		=	new UNITloader($__CONF->get('cache', 'use_apc_cache'));

			$this->BASE		=	$_SESSION['BASE'];

			$this->STOCKBASE	=	'ebagstock';
		}

		private function lock()	{

			$this->SQL->query("LOCK TABLES	$this->BASE.addressbook WRITE, 
							$this->BASE.orders_header WRITE, 
							$this->BASE.orders_data WRITE, 
							$this->STOCKBASE.finished_goods_stock WRITE, 
							ebag.companydata WRITE	");
		}

		private function unlock()	{

			$this->SQL->query('UNLOCK TABLES');
		}

		private function prepare($DATA)	{

			# note: the prices will be rereaded from base as check and prevent change in meantime

			$CODES		=	$DATA['code'];
			$NAMES		=	$DATA['name'];
			$QUANTITYS	=	$DATA['quantity'];
			$UNITS		=	$DATA['unit'];

			$LENGTH	=	count($CODES);

			if (! $LENGTH) return 'usererror';

			$RETURN	=	array();

			for ($i=0; $i<$LENGTH; $i++)	{

				$code		=	$CODES[$i];
				$name		=	$NAMES[$i];
				$quantity	=	$QUANTITYS[$i];
				$unit		=	$UNITS[$i];

				# empty rows

				if ((is_empty($code)) && (is_empty($name)) && (is_empty($quantity)) && (is_empty($unit))) continue;
				
				# Initialize vars
				
				if (! isset($RETURN['code'])) 		$RETURN['code'] = array();
				if (! isset($RETURN['name'])) 		$RETURN['name'] = array();
				if (! isset($RETURN['quantity'])) 	$RETURN['quantity'] = array();
				if (! isset($RETURN['unit'])) 		$RETURN['unit'] = array();
				if (! isset($RETURN['price'])) 		$RETURN['price'] = array();
				if (! isset($RETURN['tax'])) 		$RETURN['tax'] = array();

				# user input

				if ((is_empty($code)) or (is_empty($name))) $RETURN = 'usererror';

				if ($this->UNIT->is_int_unit($unit))	{

					if (! is_int_positive($quantity)) $RETURN = 'usererror';
						
				}	else	 {

					if (! is_real_positive($quantity)) $RETURN = 'usererror';
				}

				if ($RETURN == 'usererror') break;

				# service or artical exists

				$code	=	$RETURN['code'][]	=	$this->SQL->n_escape($code);

				$this->SQL->query("	SELECT `price`, `tax` 
							FROM $this->STOCKBASE.finished_goods_stock 
							WHERE `code`=$code AND `deleted` IS NOT NULL LIMIT 1	");

				if (! $this->SQL->num_rows())	{

					$RETURN = 'articalnotexist';

					break;
				}

				$RESULT	=	$this->SQL->RESULT->fetch_row();
				
				$RETURN['name'][]	=	$this->SQL->n_escape($name);

				# note: the quantity is not escaped because of stockchange after

				$RETURN['quantity'][]	=	$quantity;
				$RETURN['unit'][]	=	$this->SQL->n_escape($unit);
				$RETURN['price'][]	=	$this->SQL->nr_escape($RESULT[0]);
				$RETURN['tax'][]	=	$this->SQL->nr_escape($RESULT[1]);
			}
	
			if ((is_array($RETURN)) && (! count($RETURN))) $RETURN = 'usererror';

			return $RETURN;
		}

		private function check4emptyarray($DATA)	{

			$empty  = TRUE;

			$length = count($DATA['code']);

			for ($i=0; $i<$length; $i++)	{

				if (! (is_empty($DATA['code'][$i]) && is_empty($DATA['name'][$i]) &&  is_empty($DATA['quantity'][$i]) && is_empty($DATA['unit'][$i])))	{$empty = FALSE; break;}
			}
			
			return $empty;
		}

		public function update($ID, $DATA)	{

			# prepare & check header data

			if 	(((is_empty($DATA['contactID'])) && (is_empty($DATA['contactname']))) or 

				(is_empty($DATA['deadline_days'])) or

				(! is_int_positive($DATA['deadline_days']))) return 'usererror';

			$ID			=	$this->SQL->escape($ID);

			$contactID		=	$this->SQL->n_escape($DATA['contactID']);

			$contactname		=	$this->SQL->n_escape($DATA['contactname']);

			$deadline_days		=	$DATA['deadline_days'];

			# note: the payment form will not be sent if the header was locked

			if (isset($DATA['payment_form'])) $payment_form  = $this->SQL->n_escape($DATA['payment_form']);

			if (isset($DATA['delivery_form'])) $delivery_form = $this->SQL->n_escape($DATA['delivery_form']);

			$comment		=	$this->SQL->n_escape($DATA['comment']);

			$annotation		=	$this->SQL->n_escape($DATA['annotation']);

			if (! isset($DATA['use_session_discount']))	$DATA['use_session_discount'] = '';

			$use_session_discount	=	$this->SQL->n_escape($DATA['use_session_discount']);

			$session_discount	=	$this->SQL->n_escape($DATA['session_discount']);

			if (! isset($DATA['use_article_discount']))	$DATA['use_article_discount'] = '';

			$use_article_discount	=	$this->SQL->n_escape($DATA['use_article_discount']);

			# note: discounts will reread from base

			# start transaction

			$this->SQL->transaction();
		
			$this->lock();

			# read & check contact

			if ($contactID == 'NULL')	{

				$this->SQL->query("	SELECT `personID` 
							FROM $this->BASE.addressbook 
							WHERE `contactname`=$contactname LIMIT 1	");

				if ($this->SQL->num_rows())	{

					$RESULT	=	$this->SQL->RESULT->fetch_row();

					$contactID	=	$RESULT[0];
				}	
			}

			if ($contactname == 'NULL')	{

				$this->SQL->query("	SELECT `personID` 
							FROM $this->BASE.addressbook 
							WHERE `personID`=$contactID LIMIT 1	");

				if ($this->SQL->num_rows())	{

					$RESULT	=	$this->SQL->RESULT->fetch_row();

					$contactname	=	$this->SQL->nr_escape($RESULT[0]);
				}	
			}

			$this->SQL->query("	SELECT `personID` 
						FROM $this->BASE.addressbook 
						WHERE `personID`=$contactID AND `contactname`=$contactname LIMIT 1	");

			if (! $this->SQL->num_rows())	{

				$this->SQL->rollback();

				$this->unlock();

				return 'contactnotmatch';
			}
			
			if (isset($DATA['headerlock'])) $headerlock = 'set';

			# prepare service data

			# note: if no codes sent then every row was locked

			if (isset($DATA['code']))	{
			
				# note: empty array can be near lockeds
				#	every other is empty (deleted)
				
				if ($this->check4emptyarray($DATA) && isset($headerlock)) {
				
					unset($DATA['code']);
					
				}	else	  {
				
					$DATA = $this->prepare($DATA);

					if ((! isset($DATA['code'])) or (! $LENGTH = count($DATA['code']))) $DATA = 'usererror';

					if (! is_array($DATA)) 	{

						$this->SQL->rollback();

						$this->unlock();

						return $DATA;
					}
				}
			}

			# note: tax, exchange rate, currency and companyID will stay

			# read original date of creation from base also check the order is exists yet

			$this->SQL->query("	SELECT `date_of_creation` 
						FROM $this->BASE.orders_header 
						WHERE `personID`='$ID' AND `deleted` IS NOT NULL LIMIT 1	");

			if (! $this->SQL->num_rows())	{

				$this->SQL->rollback();

				$this->unlock();

				return 'unexistentorder';

			}

			$RESULT			=	$this->SQL->RESULT->fetch_row();

			$date_of_creation	=	$RESULT[0];

			$date_of_deadline	=	date("Y-m-d", strtotime($deadline_days.' days'));

			if (! $date_of_deadline) return 'dateerror';

			# note: if the deadline_days is positive the date of deadline will be older than creation

			# read actual contact address

			$this->SQL->query("	SELECT	`country`, `city`, `zip`, `street`, `delivery_country`, `delivery_city`, `delivery_street`, 
							`delivery_zip`, `article_discount`, `account1`, `bank1`, `account2`, 
							`bank2`, `delivery_day_of_week`, `contacttype`, `seller` 
						FROM $this->BASE.addressbook WHERE `personID`=$contactID AND `deleted` IS NOT NULL LIMIT 1	");

			# note: must to be row because after lock was checked before

			$RESULT	=	$this->SQL->RESULT->fetch_assoc();

			foreach ($RESULT as $key=>$val)	$RESULT[$key]=$this->SQL->nr_escape($val);

			extract($RESULT); unset($RESULT);

			# update order header depending of header was locked or not

			if (isset($headerlock))		{

				$this->SQL->query("	UPDATE $this->BASE.orders_header
							SET `comment`=$comment, `annotation`=$annotation, `date_of_deadline`='$date_of_deadline'
							WHERE `personID`='$ID'	");
			}	else	   {

				$this->SQL->query("	UPDATE $this->BASE.orders_header
							SET 	`contactID`=$contactID, `contactname`=$contactname, `country`=$country, `city`=$city, `zip`=$zip,
								`street`=$street, `delivery_country`=$delivery_country, `delivery_city`=$delivery_city,
								`delivery_street`=$delivery_street, `delivery_zip`=$delivery_zip, `article_discount`=$article_discount,
								`account1`=$account1, `bank1`=$bank1, `account2`=$account2, `bank2`=$bank2,
								`delivery_day_of_week`=$delivery_day_of_week, `contacttype`=$contacttype, `seller`=$seller, 
								`comment`=$comment, `annotation`=$annotation, `use_session_discount`=$use_session_discount,
								`use_article_discount`=$use_article_discount, `delivery_form`=$delivery_form, `payment_form`=$payment_form,
								`date_of_deadline`='$date_of_deadline'
							WHERE `personID`='$ID' LIMIT 1	");
			}

			# write back quantitys into stock for rows which are not locked 

			$this->SQL->query("	SELECT `code`, `quantity` 
						FROM $this->BASE.orders_data
						WHERE `orderID`='$ID' AND `date_of_delivery` IS NULL AND `productionorderID` IS NULL AND `deleted` IS NOT NULL	");

			if ($this->SQL->num_rows())	{

				$this->SQL->IT1_RESULT = $this->SQL->RESULT;

				while ($RESULT = $this->SQL->IT1_RESULT->fetch_assoc())	{

					$code		=	$this->SQL->nr_escape($RESULT['code']);
					$quantity	=	$RESULT['quantity'];

					$this->SQL->query("	UPDATE $this->STOCKBASE.finished_goods_stock 
								SET quantity_ordered=quantity_ordered-$quantity
								WHERE `code`=$code LIMIT 1	");
				}

				unset($this->SQL->IT1_RESULT);
			}

			# delete rows which are not locked 

			$this->SQL->query("	DELETE FROM $this->BASE.orders_data 
						WHERE `orderID`='$ID' AND `date_of_delivery` IS NULL AND `productionorderID` IS NULL	");

			# if no codes was sent step over data write

			if (isset($DATA['code']))	{
			
				$LENGTH	= count($DATA['code']);

				for ($i=0; $i<$LENGTH; $i++)	{

					$code		=	$DATA['code'][$i];
					$articalname	=	$DATA['name'][$i];
					$quantity	=	$DATA['quantity'][$i];
					$unit		=	$DATA['unit'][$i];
					$price		=	$DATA['price'][$i];
					$tax		=	$DATA['tax'][$i];

					# insert order data

					$this->SQL->query("	INSERT INTO $this->BASE.orders_data 
								VALUES  
								(NULL, '$ID', $code, $articalname, '$quantity', $unit, $price, $tax, NULL, NULL, NULL, '0')"	);

					# change quantity in stock

					$this->SQL->query("	UPDATE $this->STOCKBASE.finished_goods_stock 
								SET quantity_ordered=quantity_ordered+$quantity 
								WHERE `code`=$code AND `deleted` IS NOT NULL LIMIT 1	");

					# check stock state and set to order

					if (! $this->ORDER_TO_STOCK)	{

						$this->SQL->query("	SELECT	`quantity`, `quantity_ordered_to_stock`, `quantity_on_opened_order_to_stock`,
										`quantity_ordered`, `quantity_as_part`, `origin`
									FROM $this->STOCKBASE.finished_goods_stock WHERE `code`=$code AND `deleted` IS NOT NULL LIMIT 1	");

						# note: origin 1 is product which will not be ordered

						if ($this->SQL->num_rows())	{

							$RESULT	=	$this->SQL->RESULT->fetch_assoc();

							if ($RESULT['origin'] != 1)	{

								if (($RESULT['quantity'] + $RESULT['quantity_ordered_to_stock'] + $RESULT['quantity_on_opened_order_to_stock'] - $RESULT['quantity_ordered'] - $RESULT['quantity_as_part']) < 0) $this->ORDER_TO_STOCK = TRUE;
							}
						}
					}
				}
			}
			
			# check delivery state and update in header (also check maybe empty order was left)

			$this->SQL->query("	SELECT COUNT(*) FROM $this->BASE.orders_data 
						WHERE `orderID`='$ID' AND `date_of_delivery` IS NULL AND `deleted` IS NOT NULL	");

			$RESULT	= $this->SQL->RESULT->fetch_row();

			$not_delivered	=	$RESULT[0];

			$this->SQL->query("	SELECT COUNT(*) FROM $this->BASE.orders_data 
						WHERE `orderID`='$ID' AND `deleted` IS NOT NULL	");

			$RESULT	= $this->SQL->RESULT->fetch_row();

			$num_rows	=	$RESULT[0];

			if ($num_rows == 0)	{

				$this->SQL->rollback();

				$this->unlock();

				return 'emptyorder';
			}

			# note: deliverys states are 0 => not delivered, 1=>delivered, 2=>partially delivered

			if (! $not_delivered)	{

				$DELIVERED = 1;

			}	else 	 {

				if ($not_delivered==$num_rows) {$DELIVERED=0;} else {$DELIVERED = 2;}
			}

			$this->SQL->query("	UPDATE $this->BASE.orders_header 
						SET `delivered`='$DELIVERED' 
						WHERE `personID`='$ID' AND `deleted` IS NOT NULL LIMIT 1	");


			$this->SQL->commit();

			$this->unlock();

			return TRUE;	
		}
	}
