<?php
	/**
	 *	eBag
	 *
	 *	Open source company administration manager
	 *
	 *	@package	eBag
	 *	@author		Eper Kalman
	 *	@copyright	Copyright (c) 2012-2015, Eper Kalman
	 *	@license	GNU Affero General Public License Version 3 (http://www.gnu.org/licenses/agpl-3.0.html)
	 *	@since		Version 5.0.0
	 */

	/**
	 * eBag order write
	 */
	 
	if (! defined('eBag'))	{
	
	      include('../views/logout.php'); 
	      
	      exit();
	}

	load_f('is_real_positive');

	load_f('is_int_positive');

	class ORDERWRITE	{

		var 	$ORDER_TO_STOCK		=	FALSE;

		function __construct()	{

			global $__SQL, $__CONF;

			$this->SQL		=	&$__SQL;

			$this->CONF		=	&$__CONF;

			$this->UNIT		=	new UNITloader($__CONF->get('cache', 'use_apc_cache'));

			$this->BASE		=	$_SESSION['BASE'];

			$this->STOCKBASE	=	'ebagstock';
		}

		public function getid()		{

			if (isset($this->orderID)) {return $this->orderID;} else {return FALSE;}
		}

		private function lock()	{

			$this->SQL->query("LOCK TABLES	$this->BASE.addressbook WRITE, 
							$this->BASE.orders_header WRITE, 
							$this->BASE.orders_data WRITE, 
							$this->STOCKBASE.finished_goods_stock WRITE, 
							ebag.companydata WRITE	");
		}
	
		private function unlock()	{

			$this->SQL->query('UNLOCK TABLES');
		}

		private function prepare($DATA)	{

			# note: the prices will be rereaded from base as check and prevent change in meantime

			$CODES		=	$DATA['code'];
			$NAMES		=	$DATA['name'];
			$QUANTITYS	=	$DATA['quantity'];
			$UNITS		=	$DATA['unit'];

			$LENGTH	=	count($CODES);

			if (! $LENGTH) return 'usererror';

			$RETURN	=	array();

			for ($i=0; $i<$LENGTH; $i++)	{

				$code		=	$CODES[$i];
				$name		=	$NAMES[$i];
				$quantity	=	$QUANTITYS[$i];
				$unit		=	$UNITS[$i];

				# empty rows

				if ((is_empty($code)) && (is_empty($name)) && (is_empty($quantity)) && (is_empty($unit))) continue;
				
				# Initialize vars
				
				if (! isset($RETURN['code'])) 		$RETURN['code'] = array();
				if (! isset($RETURN['name'])) 		$RETURN['name'] = array();
				if (! isset($RETURN['quantity'])) 	$RETURN['quantity'] = array();
				if (! isset($RETURN['unit'])) 		$RETURN['unit'] = array();
				if (! isset($RETURN['price'])) 		$RETURN['price'] = array();
				if (! isset($RETURN['tax'])) 		$RETURN['tax'] = array();

				# user input

				if ((is_empty($code)) or (is_empty($name))) $RETURN = 'usererror';

				if ($this->UNIT->is_int_unit($unit))	{

					if (! is_int_positive($quantity)) $RETURN = 'usererror';
						
				}	else	 {

					if (! is_real_positive($quantity)) $RETURN = 'usererror';
				}

				if ($RETURN == 'usererror') break;

				# service or artical exists

				$code	=	$RETURN['code'][]	=	$this->SQL->n_escape($code);

				$this->SQL->query("	SELECT `price`, `tax` 
							FROM $this->STOCKBASE.finished_goods_stock 
							WHERE `code`=$code AND `deleted` IS NOT NULL LIMIT 1	");

				if (! $this->SQL->num_rows())	{

					$RETURN = 'articalnotexist';

					break;
				}

				$RESULT	=	$this->SQL->RESULT->fetch_row();
				
				$RETURN['name'][]	=	$this->SQL->n_escape($name);

				# note: the quantity is not escaped because of stockchange after

				$RETURN['quantity'][]	=	$quantity;
				$RETURN['unit'][]	=	$this->SQL->n_escape($unit);
				$RETURN['price'][]	=	$this->SQL->nr_escape($RESULT[0]);
				$RETURN['tax'][]	=	$this->SQL->nr_escape($RESULT[1]);
			}
	
			if ((is_array($RETURN)) && (! count($RETURN))) $RETURN = 'usererror';

			return $RETURN;
		}

		public function write($DATA)	{

			# prepare & check header data

			if 	(((is_empty($DATA['contactID'])) && (is_empty($DATA['contactname']))) or 

				(is_empty($DATA['deadline_days'])) or

				(! is_int_positive($DATA['deadline_days'])) or 

				(is_empty($DATA['payment_form'])) or

				(is_empty($DATA['delivery_form']))) return 'usererror';

			$contactID		=	$this->SQL->n_escape($DATA['contactID']);

			$contactname		=	$this->SQL->n_escape($DATA['contactname']);

			$deadline_days		=	$this->SQL->n_escape($DATA['deadline_days']);

			$payment_form		=	$this->SQL->n_escape($DATA['payment_form']);

			$delivery_form		=	$this->SQL->n_escape($DATA['delivery_form']);

			$comment		=	$this->SQL->n_escape($DATA['comment']);

			$annotation		=	$this->SQL->n_escape($DATA['annotation']);

			if (! isset($DATA['use_session_discount']))	$DATA['use_session_discount'] = '';

			$use_session_discount	=	$this->SQL->n_escape($DATA['use_session_discount']);

			$session_discount	=	$this->SQL->n_escape($DATA['session_discount']);

			if (! isset($DATA['use_article_discount']))	$DATA['use_article_discount'] = '';

			$use_article_discount	=	$this->SQL->n_escape($DATA['use_article_discount']);

			# note: discounts will reread from base

			$date_of_creation	=	date("Y-m-d");

			$date_of_deadline	=	date("Y-m-d", strtotime($DATA['deadline_days'].' days'));

			if (! $date_of_deadline) return 'dateerror';

			# note: if the deadline_days is positive the date of deadline will be older than creation

			# start transaction

			$this->SQL->transaction();
		
			$this->lock();

			# read & check contact

			if ($contactID == 'NULL')	{

				$this->SQL->query("	SELECT `personID` 
							FROM $this->BASE.addressbook 
							WHERE `contactname`=$contactname LIMIT 1	");

				if ($this->SQL->num_rows())	{

					$RESULT	=	$this->SQL->RESULT->fetch_row();

					$contactID	=	$RESULT[0];
				}	
			}

			if ($contactname == 'NULL')	{

				$this->SQL->query("	SELECT `personID` 
							FROM $this->BASE.addressbook 
							WHERE `personID`=$contactID LIMIT 1	");

				if ($this->SQL->num_rows())	{

					$RESULT	=	$this->SQL->RESULT->fetch_row();

					$contactname	=	$this->SQL->nr_escape($RESULT[0]);
				}	
			}

			$this->SQL->query("	SELECT `personID` 
						FROM $this->BASE.addressbook 
						WHERE `personID`=$contactID AND `contactname`=$contactname LIMIT 1	");

			if (! $this->SQL->num_rows())	{

				$this->SQL->rollback();

				$this->unlock();

				return 'contactnotmatch';
			}

			# prepare service data
			
			$DATA = $this->prepare($DATA);

			if ((! isset($DATA['code'])) or (! $LENGTH = count($DATA['code']))) $DATA = 'usererror';

			if (! is_array($DATA)) 	{

				$this->SQL->rollback();

				$this->unlock();

				return $DATA;
			}

			# set & read essentials

			$exchange_rate	=	$this->SQL->n_escape($this->CONF->get('payments', 'exchange_rate'));

			$currency	=	$this->SQL->n_escape($this->CONF->get('payments', 'currency'));
			
			$valuta		=	$this->SQL->n_escape($this->CONF->get('payments', 'valuta'));

			$companyID	=	FALSE;

			$this->SQL->query("	SELECT MAX(`personID`) AS `personID` 
						FROM ebag.companydata	");

			if ($this->SQL->num_rows())	{

				$RESULT		=	$this->SQL->RESULT->fetch_row();
				$companyID	=	$RESULT[0];
			}

			if (($exchange_rate == 'NULL') or ($currency == 'NULL') or ($valuta == 'NULL') or (! $companyID))	{

				$this->SQL->rollback();

				$this->unlock();

				if (! $companyID) {return 'nocompanyID';} else {return 'fatalerror';}
			}

			$companyID	=	"'$companyID'";

			# read actual contact address

			$this->SQL->query("	SELECT	`country`, `city`, `zip`, `street`, `delivery_country`, `delivery_city`, `delivery_street`, 
							`delivery_zip`, `article_discount`, `account1`, `bank1`, `account2`, 
							`bank2`, `delivery_day_of_week`, `contacttype`, `seller`, `critic` 
						FROM $this->BASE.addressbook WHERE `personID`=$contactID AND `deleted` IS NOT NULL LIMIT 1	");

			# note: must to be row because after lock was checked before

			$RESULT	=	$this->SQL->RESULT->fetch_assoc();

			# critic customer cannot make order
			# note:	critic 0 or 1

			if ($RESULT['critic'])	{

				$this->SQL->rollback();

				$this->unlock();

				return 'criticalcustomer';
			}

			foreach ($RESULT as $key=>$val)	$RESULT[$key]=$this->SQL->nr_escape($val);

			extract($RESULT); unset($RESULT);

			# insert order header

			$this->SQL->query("	INSERT INTO  $this->BASE.orders_header VALUES (	NULL, $companyID, $contactID, $contactname, $country, $city, $zip, $street,
												$delivery_country, $delivery_city, $delivery_street, $delivery_zip,
												$article_discount, $account1, $bank1, $account2, $bank2, $delivery_day_of_week,
												$contacttype, $seller, $comment, $annotation, $currency, $valuta, $exchange_rate, 
												$session_discount, $use_session_discount, $use_article_discount, $delivery_form,
												$payment_form, NULL, '$date_of_creation', '$date_of_deadline', '0', NULL, '0')	");
			$this->orderID	=	$this->SQL->lastid();

			if (! $this->orderID)	{
			
				$this->SQL->rollback();

				$this->unlock();

				return 'unknownerror';
			}

			$LENGTH		=	count($DATA['code']);

			for ($i=0; $i<$LENGTH; $i++)	{

				$code		=	$DATA['code'][$i];
				$articalname	=	$DATA['name'][$i];
				$quantity	=	$DATA['quantity'][$i];
				$unit		=	$DATA['unit'][$i];
				$price		=	$DATA['price'][$i];
				$tax		=	$DATA['tax'][$i];
				
				# insert order data

				$this->SQL->query("	INSERT INTO $this->BASE.orders_data 
							VALUES  
							(NULL, '$this->orderID', $code, $articalname, '$quantity', $unit, $price, $tax, NULL, NULL, NULL, '0')");

				# change quantity in stock

				$this->SQL->query("	UPDATE $this->STOCKBASE.finished_goods_stock 
							SET quantity_ordered=quantity_ordered+$quantity 
							WHERE `code`=$code AND `deleted` IS NOT NULL LIMIT 1	");

				# check stock state and set to order

				if (! $this->ORDER_TO_STOCK)	{

					$this->SQL->query("	SELECT	`quantity`, `quantity_ordered_to_stock`, `quantity_on_opened_order_to_stock`,
									`quantity_ordered`, `quantity_as_part`, `origin`
								FROM $this->STOCKBASE.finished_goods_stock 
								WHERE `code`=$code AND `deleted` IS NOT NULL LIMIT 1	");

					# note: origin 1 is product which will not be ordered

					if ($this->SQL->num_rows())	{

						$RESULT	=	$this->SQL->RESULT->fetch_assoc();

						if ($RESULT['origin'] != 1)	{

							if (($RESULT['quantity'] + $RESULT['quantity_ordered_to_stock'] + $RESULT['quantity_on_opened_order_to_stock'] - $RESULT['quantity_ordered'] - $RESULT['quantity_as_part']) < 0) $this->ORDER_TO_STOCK = TRUE;
						}
					}
				}
			}

			$this->SQL->commit();

			$this->unlock();

			return TRUE;	
		}
	}
?>